;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Instance <Set_T> of the theory FiniteSetWithCard
;; Set_T-template.smt

 :extrasorts (<Set_T>)

 :extrafuns  ((embed_<Set_T>__Array <Set_T> Array)
              (disembed_<Set_T>__Array Array <Set_T>))
 :assumption (forall (?x <Set_T>)
                     (= (disembed_<Set_T>__Array (embed_<Set_T>__Array ?x)) ?x)
                     :pat { (embed_<Set_T>__Array ?x) })

 :assumption (forall (?x <Set_T>) (?y Int)
                     (or (= (select (embed_<Set_T>__Array ?x) ?y) Set_elem)
                         (= (select (embed_<Set_T>__Array ?x) ?y) Set_nonelem))
                     :pat { (select (embed_<Set_T>__Array ?x) ?y) })

;; Empty sets

 :extrapreds ((isEmpty_<Set_T> <Set_T>))
 :assumption (forall (?x <Set_T>)
                     (iff (isEmpty_<Set_T> ?x)
                          (forall (?y Int)
                                  (= (select (embed_<Set_T>__Array ?x) ?y) Set_nonelem)))
              :pat { (isEmpty_<Set_T> ?x) } )

 :extrafuns ((emptySet_<Set_T> <Set_T>))
 :assumption (isEmpty_<Set_T> emptySet_<Set_T>)

;; Set insertion

 :extrafuns ((insert_<Set_T> <T> <Set_T> <Set_T>))
 :assumption (forall (?x <T>) (?y <Set_T>)
                     (= (embed_<Set_T>__Array (insert_<Set_T> ?x ?y))
                        (store (embed_<Set_T>__Array ?y)
                               (embed_<T>__Int ?x) Set_elem))
                     :pat { (insert_<Set_T> ?x ?y) } )

;; Set membership predicate

 :extrapreds ((in_<Set_T> <T> <Set_T>))
 :assumption (forall (?x <T>) (?y <Set_T>)
                     (iff (in_<Set_T> ?x ?y)
                          (= (select (embed_<Set_T>__Array ?y)
                                     (embed_<T>__Int ?x))
                             Set_elem))
                     :pat { (in_<Set_T> ?x ?y) } )

;; Subset relation

 :extrapreds ((subset_<Set_T> <Set_T> <Set_T>))
 :assumption (forall (?x <Set_T>) (?y <Set_T>)
                     (iff (subset_<Set_T> ?x ?y)
                          (forall (?z Int)
                                  (implies (= (select (embed_<Set_T>__Array ?x) ?z)
                                              Set_elem)
                                           (= (select (embed_<Set_T>__Array ?y) ?z)
                                              Set_elem))))
                     :pat { (subset_<Set_T> ?x ?y) } )

;; Union of sets

 :extrafuns ((union_<Set_T> <Set_T> <Set_T> <Set_T>))
 :assumption (forall (?x Int) (?y <Set_T>) (?z <Set_T>)
                     (iff (= (select (embed_<Set_T>__Array (union_<Set_T> ?y ?z)) ?x)
                             Set_elem)
                          (or (= (select (embed_<Set_T>__Array ?y) ?x) Set_elem)
                              (= (select (embed_<Set_T>__Array ?z) ?x) Set_elem)))
                     :pat { (select (embed_<Set_T>__Array (union_<Set_T> ?y ?z)) ?x) } )

 :assumption (= (union_<Set_T> emptySet_<Set_T> emptySet_<Set_T>)
                emptySet_<Set_T>)

;; Intersection of sets

 :extrafuns ((inter_<Set_T> <Set_T> <Set_T> <Set_T>))
 :assumption (forall (?x Int) (?y <Set_T>) (?z <Set_T>)
                     (iff (= (select (embed_<Set_T>__Array (inter_<Set_T> ?y ?z)) ?x)
                             Set_elem)
                          (and (= (select (embed_<Set_T>__Array ?y) ?x) Set_elem)
                               (= (select (embed_<Set_T>__Array ?z) ?x) Set_elem)))
                     :pat { (select (embed_<Set_T>__Array (inter_<Set_T> ?y ?z)) ?x) } )

;; Difference of sets

 :extrafuns ((setminus_<Set_T> <Set_T> <Set_T> <Set_T>))
 :assumption (forall (?x Int) (?y <Set_T>) (?z <Set_T>)
                     (iff (= (select (embed_<Set_T>__Array (setminus_<Set_T> ?y ?z)) ?x)
                             Set_elem)
                          (and (= (select (embed_<Set_T>__Array ?y) ?x) Set_elem)
                               (= (select (embed_<Set_T>__Array ?z) ?x) Set_nonelem)))
                     :pat { (select (embed_<Set_T>__Array (setminus_<Set_T> ?y ?z)) ?x) } )

;; Cardinality of sets, with various rules how to derive cardinalities

 :extrafuns ((card_<Set_T> <Set_T> Int))

 :assumption (= (card_<Set_T> emptySet_<Set_T>) 0)

 :assumption (forall (?x <Set_T>) (>= (card_<Set_T> ?x) 0)
              :pat { (card_<Set_T> ?x) } )

 :assumption (forall (?x <T>) (?y <Set_T>)
                     (= (card_<Set_T> (insert_<Set_T> ?x ?y))
                        (ite (in_<Set_T> ?x ?y) (card_<Set_T> ?y)
                                              (+ (card_<Set_T> ?y) 1)))
              :pat { (card_<Set_T> (insert_<Set_T> ?x ?y)) } )

 :assumption (forall (?x <Set_T>) (?y <Set_T>)
                     (implies (subset_<Set_T> ?x ?y)
                              (<= (card_<Set_T> ?x) (card_<Set_T> ?y)))
              :pat { (subset_<Set_T> ?x ?y) } )

 :assumption (forall (?x <Set_T>) (?y <Set_T>)
                     (= (card_<Set_T> (union_<Set_T> ?x ?y))
                        (- (+ (card_<Set_T> ?x) (card_<Set_T> ?y))
                           (card_<Set_T> (inter_<Set_T> ?x ?y)))))

 :assumption (forall (?x <T>) (?y <Set_T>) (?z <Set_T>)
                     (= (union_<Set_T> (insert_<Set_T> ?x ?y) ?z)
                        (insert_<Set_T> ?x (union_<Set_T> ?y ?z)))
              :pat { (card_<Set_T> (union_<Set_T> (insert_<Set_T> ?x ?y) ?z)) } )

 :assumption (forall (?x <T>) (?y <Set_T>) (?z <Set_T>)
                     (= (union_<Set_T> ?z (insert_<Set_T> ?x ?y))
                        (insert_<Set_T> ?x (union_<Set_T> ?z ?y)))
              :pat { (card_<Set_T> (union_<Set_T> ?z (insert_<Set_T> ?x ?y))) } )

 :assumption (forall (?x <Set_T>) (?y <Set_T>)
                     (= (card_<Set_T> (setminus_<Set_T> ?x ?y))
                        (- (card_<Set_T> ?x) (card_<Set_T> (inter_<Set_T> ?x ?y))))
              : pat { (card_<Set_T> (setminus_<Set_T> ?x ?y)) })

;; End: Instance <Set_T> of the theory FiniteSetWithCard
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
