/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Class to read, instantiate, and output snippets of some output format
 * (here in particular used to instantiate SMT-LIB axioms)
 */

#include "file-template.H"

#include <sstream>
#include <fstream>
#include <cassert>

using namespace std;

FileTemplate::FileTemplate(const string& filename)
    : filename (filename), contents(new vector<string>),
      loaded(new bool (false)) {}

void FileTemplate::load() const {
    ifstream file (filename.c_str());

    if (!file.is_open())
        throw IOException("File " + filename + " could not be opened");

    string line;
    while (!file.eof()) {
        getline(file, line);
        contents->push_back(line);
    }

    file.close();

    *loaded = true;
}

void FileTemplate::instantiate(std::ostream& os,
                               const StringSubstitution& subst) const {
    if (!*loaded)
        load();
    for (vector<string>::const_iterator it = contents->begin();
         it != contents->end();
         ++it)
        os << replace(*it, subst) << endl;
}

std::string FileTemplate::replace(const std::string& line,
                                  const std::string& find,
                                  const std::string& replace) const {
    stringstream res;

    for (string::size_type i = 0; i < line.size(); ) {
        const string::size_type pos = line.find(find, i);

        res << line.substr(i, pos - i);
        if (pos < line.size()) {
            res << replace;
            i = pos + find.size();
        } else {
            i = pos;
        }
    }

    return res.str();
}

std::string FileTemplate::replace(const std::string& line,
                                  const StringSubstitution& subst) const {
    string str = line;
    for (StringSubstitution::const_iterator it = subst.begin();
         it != subst.end();
         ++it)
        str = replace(str, it->first, it->second);

    return str;
}

IOException::IOException(const std::string& message)
    : message(message) {}

IOException::~IOException() throw () {}

const char* IOException::what() const throw() {
    return message.c_str();
}
