/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <vector>
#include <cstdio>
#include <string>

#include "smtlib-absyn.H"
#include "util.H"
#include "smtlib-resolver.H"
#include "smt-back-end/translator.H"
#ifdef WIN32
#  include "smt-back-end/file-templates.H"
#endif

using namespace std;
using namespace SMTLibAbsyn;
using namespace Resolver;
#ifdef WIN32
  using namespace SMTBackend;
#endif

int main(int argc, char ** argv) {

  cerr << "The SMT-LIB 2 pre-processor, version " << VERSION << endl;
  cerr << "(http://www.cprover.org/SMT-LIB-LSM)" << endl;

#ifdef WIN32
  // we need to find out about the path of the executable,
  // so that we later can load the SMT resource files

  FileTemplates::basepath = argv[0];
  string::size_type slashPos = FileTemplates::basepath.rfind("\\");
  assert(slashPos >= 0 && slashPos < FileTemplates::basepath.size());
  FileTemplates::basepath.resize(slashPos);
  slashPos = FileTemplates::basepath.rfind("\\");
  assert(slashPos >= 0 && slashPos < FileTemplates::basepath.size());
  FileTemplates::basepath.resize(slashPos);

  FileTemplates::basepath =
    FileTemplates::basepath + "\\resources\\smt-backend\\";
#endif

  FILE *input;
  if (argc > 1) {
    input = fopen(argv[1], "r");
    if (!input) {
      cerr << "Error opening input file." << endl;
      return 1;
    }
  } else {
    cerr << "Reading from stdin ..." << endl;
    input = stdin;
  }

  BenchmarkPtr benchmark;
  try {
      benchmark = translateBenchmark(input);
  } catch (TranslationException &e) {
      cerr << "Error: " << e.what() << endl;
      return 1;
  }

//	cout << *benchmark->formula << endl;

  assert(benchmark);

  SMTBackend::FileTemplates templates;
  SMTBackend::Translator translator (templates, cout, benchmark);
  translator.translate();
}
