/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Some helper functions for sort substitutions and sort unification
 */

#ifndef SORT_UNIFICATION
#define SORT_UNIFICATION 1

#include <map>
#include <set>

#include "smtlib-absyn.H"

namespace SortUnification {

using namespace SMTLibAbsyn;

/**
 * Distinguished nullary sort constructors that denote variables. Because
 * variables are only used internally and are never presented to the user, such
 * constructors are not given names either.
 */
class SortVariable : public SortCtor {

public:
    SortVariable();

};

typedef CountingPtr<SortVariable> SortVariablePtr;
typedef std::map<SortVariablePtr, SortPtr> Substitution;

std::set<SortVariablePtr> freeVariables(const SortPtr& sort);

SortPtr substitute(const SortPtr& sort, const Substitution& subst);

bool unify(const SortPtr& a, const SortPtr& b, Substitution& unifier);

};

#endif
