/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Some classes to apply substitutions to SMT-LIB expressions and to
 * eliminate let-expressions.
 */

#ifndef SUBSTITUTIONS
#define SUBSTITUTIONS 1

#include "smtlib-absyn-visitors.H"
#include "smtlib-resolver.H"

#include <vector>

namespace Substitutions {

using namespace Visitors;
using namespace SMTLibAbsyn;

/**
 * Shift the free variables in an expression upwards or downwards. More
 * precisely, the variables with index 0, 1, 2, ..., offset, offset + 1, ...
 * are replaced with 0, 1, 2, ..., offset + shift, offset + shift + 1, ...
 */
ExpressionPtr shiftVariables(const ExpressionPtr& expr,
                             unsigned int offset, int shift);

/**
 * Eliminate let-expressions whose sort is not boolean (such expressions are
 * not supported in SMT-LIB 1)
 */
ExpressionPtr eliminateLet(const ExpressionPtr& expr);

void eliminateLet(const Resolver::BenchmarkPtr& benchmark);
}

#endif
