/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "function-visitor.H"

namespace Theories {

using namespace SMTLibAbsyn;

/**
 * The empty theory
 */
void FunctionVisitor::visit_fun_true     (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_false    (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_not      (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_implies  (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_and      (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_or       (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_xor      (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_equal(const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_ite  (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_distinct(const FunctionAppPtr& node) {
    visit_other(node);
}

/**
 * The theory of integer arithmetic
 */
void FunctionVisitor::visit_fun_unary_minus(const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_minus      (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_plus       (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_times      (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_leq        (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_less       (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_geq        (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_fun_greater    (const FunctionAppPtr& node) {
    visit_other(node);
}

/**
 * The theory of finite sets with cardinality
 */
void FunctionVisitor::visit_polyfun_emptySet(const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_insert  (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_in      (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_subset  (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_union   (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_inter   (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_setminus(const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_card    (const FunctionAppPtr& node) {
    visit_other(node);
}

/**
 * The theory of lists with length
 */
void FunctionVisitor::visit_polyfun_nil   (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_cons  (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_head  (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_tail  (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_append(const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_length(const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_nth   (const FunctionAppPtr& node) {
    visit_other(node);
}

/**
 * The theory of finite maps
 */
void FunctionVisitor::visit_polyfun_emptyMap (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_apply    (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_overwrite(const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_domain   (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_range    (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_restrict (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_subtract (const FunctionAppPtr& node) {
    visit_other(node);
}

/**
 * The theory of finite relations with cardinality
 */
void FunctionVisitor::visit_polyfun_tuple    (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_project  (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_product  (const FunctionAppPtr& node) {
    visit_other(node);
}

/**
 * The theory of lists, relations (including sets), and partial maps
 */
void FunctionVisitor::visit_polyfun_inds     (const FunctionAppPtr& node) {
    visit_other(node);
}
void FunctionVisitor::visit_polyfun_elems    (const FunctionAppPtr& node) {
    visit_other(node);
}

}
