/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "integers.H"
#include "../util.H"
#include "function-visitor.H"

namespace Theories {

using namespace SMTLibAbsyn;

Integers::Integers(const Empty& empty)
: empty(empty),
  sort_Int       (new SortCtor ("Int", 0)),
  fun_unary_minus(new MonoFunction ("-", sort_Int->m(),
                                    to_vector(sort_Int->m()))),
  fun_minus      (new MonoFunction ("-", sort_Int->m(),
                                    to_vector(sort_Int->m(),
                                              sort_Int->m()))),
  fun_plus       (new MonoFunction ("+", sort_Int->m(),
                                    to_vector(sort_Int->m(),
                                              sort_Int->m()))),
  fun_times      (new MonoFunction ("*", sort_Int->m(),
                                    to_vector(sort_Int->m(),
                                              sort_Int->m()))),
  fun_leq        (new MonoFunction ("<=", Sort::Bool,
                                    to_vector(sort_Int->m(),
                                              sort_Int->m()))),
  fun_less       (new MonoFunction ("<", Sort::Bool,
                                    to_vector(sort_Int->m(),
                                              sort_Int->m()))),
  fun_geq        (new MonoFunction (">=", Sort::Bool,
                                    to_vector(sort_Int->m(),
                                              sort_Int->m()))),
  fun_greater    (new MonoFunction (">", Sort::Bool,
                                    to_vector(sort_Int->m(),
                                              sort_Int->m()))),
  multiArityFun_minus  (new LeftAssocFunctionFamily (fun_minus)),
  multiArityFun_plus   (new LeftAssocFunctionFamily (fun_plus)),
  multiArityFun_times  (new LeftAssocFunctionFamily (fun_times)),
  multiArityFun_leq    (new ChainableFunctionFamily (fun_leq,
                                                     empty.fun_true, empty.fun_and)),
  multiArityFun_less   (new ChainableFunctionFamily (fun_less,
                                                     empty.fun_true, empty.fun_and)),
  multiArityFun_geq    (new ChainableFunctionFamily (fun_geq,
                                                     empty.fun_true, empty.fun_and)),
  multiArityFun_greater(new ChainableFunctionFamily (fun_greater,
                                                     empty.fun_true, empty.fun_and))
  {}

void Integers::registerSymbolsLocally(Environment& env) const {
    env.addSortCtorFamily(sort_Int->name.body,
                          new SimpleSortCtorFamily(sort_Int), true);
    env.addLiteralSort(Numeral, sort_Int->m());
    registerMonoFunction(env, fun_unary_minus);
    registerMultiArityFunction(env, fun_minus, multiArityFun_minus);
    registerMultiArityFunction(env, fun_plus, multiArityFun_plus);
    registerMultiArityFunction(env, fun_times, multiArityFun_times);
    registerMultiArityFunction(env, fun_leq, multiArityFun_leq);
    registerMultiArityFunction(env, fun_less, multiArityFun_less);
    registerMultiArityFunction(env, fun_geq, multiArityFun_geq);
    registerMultiArityFunction(env, fun_greater, multiArityFun_greater);
}

void Integers::registerSymbols(Environment& env) const {
    empty.registerSymbolsLocally(env);
    registerSymbolsLocally(env);
}

void Integers::visit(const FunctionAppPtr& node,
                     FunctionVisitor& visitor) const {
    if (!(visitLocally(node, visitor) || empty.visitLocally(node, visitor)))
        visitor.visit_other(node);
}

bool Integers::visitLocally(const FunctionAppPtr& node,
                            FunctionVisitor& visitor) const {
    const MonoFunctionPtr fun = node->function;
    if (fun == fun_unary_minus)
        visitor.visit_fun_unary_minus(node);
    else if (fun == fun_minus)
        visitor.visit_fun_minus(node);
    else if (fun == fun_plus)
        visitor.visit_fun_plus(node);
    else if (fun == fun_times)
        visitor.visit_fun_times(node);
    else if (fun == fun_leq)
        visitor.visit_fun_leq(node);
    else if (fun == fun_less)
        visitor.visit_fun_less(node);
    else if (fun == fun_geq)
        visitor.visit_fun_geq(node);
    else if (fun == fun_greater)
        visitor.visit_fun_greater(node);
    else {
        return false;
    }
    return true;
}

const Empty& Integers::getEmpty() const {
    return empty;
}

}
