/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The theory of finite lists with length
 */

#ifndef THEORY_LISTS
#define THEORY_LISTS 1

#include "integers.H"
#include "poly-function-family.H"

namespace Theories {

using namespace SMTLibAbsyn;

class Lists : public TheoryHelpers {
public:
    Lists(const Empty& empty, const Integers& integers);

    const Empty& empty;
    const Integers& integers;

    const SortCtorPtr           sort_List;
    const PolyFunctionFamilyPtr polyfun_nil;
    const PolyFunctionFamilyPtr polyfun_cons;
    const PolyFunctionFamilyPtr polyfun_head;
    const PolyFunctionFamilyPtr polyfun_tail;
    const PolyFunctionFamilyPtr polyfun_append;
    const PolyFunctionFamilyPtr polyfun_length;
    const PolyFunctionFamilyPtr polyfun_nth;

    const MultiArityFunctionFamilyPtr multiArityFun_cons;
    const MultiArityFunctionFamilyPtr multiArityFun_append;

    void registerSymbols(Environment& env) const;
    void registerSymbolsLocally(Environment& env) const;
    void visit(const FunctionAppPtr& node, FunctionVisitor& visitor) const;
    bool visitLocally(const FunctionAppPtr& node, FunctionVisitor& visitor) const;

    const Empty& getEmpty() const;
    static Lists* create();
};

}

#endif
