/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The theory of lists, relations (including sets), and partial maps
 */

#ifndef THEORY_LRM
#define THEORY_LRM 1

#include "integers.H"
#include "sets.H"
#include "lists.H"
#include "maps.H"
#include "relations.H"
#include "poly-function-family.H"

namespace Theories {

using namespace SMTLibAbsyn;

class LRM : public TheoryHelpers {
public:
    LRM(const Empty&     empty,
        const Integers&  integers,
		const Sets&      sets,
		const Lists&     lists,
		const Relations& relations,
		const Maps&      maps);

    const Empty&     empty;
    const Integers&  integers;
    const Sets&      sets;
    const Lists&     lists;
    const Relations& relations;
    const Maps&      maps;

    const PolyFunctionFamilyPtr polyfun_inds;
    const PolyFunctionFamilyPtr polyfun_elems;

    void registerSymbols(Environment& env) const;
    void registerSymbolsLocally(Environment& env) const;
    void visit(const FunctionAppPtr& node, FunctionVisitor& visitor) const;
    bool visitLocally(const FunctionAppPtr& node, FunctionVisitor& visitor) const;

    const Empty& getEmpty() const;
    static LRM* create();
};

}

#endif
